﻿using System;
using System.Data.OleDb;

using DevExpress.Utils;
using DevExpress.XtraEditors;
using DevExpress.XtraEditors.Controls;
using DevExpress.XtraGrid.Columns;
using DevExpress.XtraGrid.Views.Grid;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 연결
        /// </summary>
        private OleDbConnection connection;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.connection = new OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=nwind.mdb");

            this.gridView.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                true                            // Editable
            );

            this.gridView.OptionsBehavior.AutoPopulateColumns = true;

            this.gridControl.DataSource = Product.GetProductList(this.connection);

            this.gridView.BestFitColumns();

            this.gridView.ValidatingEditor      += gridView_ValidatingEditor;
            this.gridView.InvalidValueException += gridView_InvalidValueException;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 그리드 뷰 에디터 값 검증하기 - gridView_ValidatingEditor(sender, e)

        /// <summary>
        /// 그리드 뷰 에디터 값 검증하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridView_ValidatingEditor(object sender, BaseContainerValidateEditorEventArgs e)
        {
            GridView view = sender as GridView;

            GridColumn column = (e as EditFormValidateEditorEventArgs)?.Column ?? view.FocusedColumn;

            if(column.FieldName != "UnitPrice")
            {
                return;
            }

            if((Convert.ToInt32(e.Value) < 0) || (Convert.ToInt32(e.Value) > 1000))
            {
                e.Valid = false;
            }
        }

        #endregion
        #region 그리드 뷰 무효 값 예외 처리하기 - gridView_InvalidValueException(sender, e)

        /// <summary>
        /// 그리드 뷰 무효 값 예외 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridView_InvalidValueException(object sender, InvalidValueExceptionEventArgs e)
        {
            GridView gridView = sender as GridView;

            e.ExceptionMode = ExceptionMode.DisplayError;
            e.WindowCaption = "입력 에러";
            e.ErrorText     = "값은 0 이상 1000 이하여야 합니다.";

            gridView.HideEditor();
        }

        #endregion
    }
}